package app.auto;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.aopalliance.intercept.MethodInvocation;
import org.springframework.beans.factory.BeanNameAware;
import org.springframework.remoting.RemoteAccessException;
import org.springframework.remoting.httpinvoker.HttpInvokerProxyFactoryBean;

import app.service.CacheServiceInterface;
import app.service.ServicePublisher;

public class AutoDiscoveredService extends HttpInvokerProxyFactoryBean
		implements BeanNameAware {

	private String beanName;

	private CacheServiceInterface cache;

	public void setCache(CacheServiceInterface cache) {
		this.cache = cache;
	}

	public void setServiceUrl(String arg0) {
		throw new IllegalStateException(
				"cannot set url on this one, supposed to be autoconfigured");
	}

	public Object invoke(MethodInvocation arg0) throws Throwable {

		List urls = getServiceUrls();
		if (urls != null)
			for (Iterator allUrls = urls.iterator(); allUrls.hasNext();) {
				String serviceUrl = null;
				try {
					serviceUrl = (String) allUrls.next();
					super.setServiceUrl(serviceUrl);
					logger.info("going to:" + super.getServiceUrl());
					return super.invoke(arg0);
				} catch (Throwable problem) {
					if (problem instanceof IOException
							|| problem instanceof RemoteAccessException) {
						logger.warn("got error accessing:"
								+ super.getServiceUrl(), problem);
						removeFailedService(serviceUrl);
					} else {
						throw problem;
					}
				}
			}
		throw new IllegalStateException("No services configured for name:"
				+ beanName);
	}

	private void removeFailedService(String url) {
		try {
			logger.info("removing failed service:" + url);
			Set services = (Set) cache.get(ServicePublisher.SERVICE_PREFIX
					+ beanName, ServicePublisher.SERVICE_KEY);
			if (services != null) {
				services.remove(url);
				cache.put(ServicePublisher.SERVICE_PREFIX + beanName, ServicePublisher.SERVICE_KEY,
						services);
				logger.info("removed failed service at:" + url);
			}
		} catch (Exception e) {
			logger.warn("failed to remove failed service:" + url, e);
		}
	}
	
	private List getServiceUrls() throws Exception {
		Set services = (Set) cache.get(ServicePublisher.SERVICE_PREFIX
				+ beanName, ServicePublisher.SERVICE_KEY);
		if (services == null)
			return null;
		ArrayList results = new ArrayList(services);
		Collections.shuffle(results);
		logger.info("shuffled:" + results);
		return results;
	}

	public void setBeanName(String arg0) {
		beanName = arg0;
	}

}
