package app.service;

import java.net.InetAddress;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.ApplicationEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.context.event.ContextClosedEvent;
import org.springframework.context.event.ContextRefreshedEvent;

public class ServicePublisher implements ApplicationListener,
		ApplicationContextAware {

	public static final String SERVICE_PREFIX = "/auto/services/";

	public static final String SERVICE_KEY = "key";

	private ApplicationContext context;

	private CacheServiceInterface cache;

	private Logger logger = Logger.getLogger(ServicePublisher.class);

	private String localIpAddress;

	private String localPort = "8080";

	private String urlQueryPrefix = "/jboss-cache-spring-disc/remote-service";

	public void setCache(CacheServiceInterface cache) {
		this.cache = cache;
	}

	public void onApplicationEvent(ApplicationEvent event) {
		try {
			if (event instanceof ContextRefreshedEvent) {
				contextRefreshed();
			}

			if (event instanceof ContextClosedEvent) {
				contextClosed();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void contextClosed() throws Exception {
		logger.info("context closed");
		String[] names = context
				.getBeanNamesForType(AutoDiscoveredServiceExporter.class);
		logger.info("closing services:" + names.length);
		for (int i = 0; i < names.length; i++) {
			String serviceUrl = makeUrl(names[i]);
			try {
				Set services = (Set) cache.get(SERVICE_PREFIX + names[i],
						SERVICE_KEY);
				if (services != null) {
					services.remove(serviceUrl);
					cache.put(SERVICE_PREFIX + names[i], SERVICE_KEY, services);
				}
				logger.info("removed:" + serviceUrl);
			} catch (Exception ex) {
				logger.error("exception removing service:" + serviceUrl, ex);
			}
		}

	}

	private void contextRefreshed() throws Exception {
		logger.info("context refreshed");

		String[] names = context
				.getBeanNamesForType(AutoDiscoveredServiceExporter.class);
		logger.info("exporting services:" + names.length);
		for (int i = 0; i < names.length; i++) {
			String serviceUrl = makeUrl(names[i]);
			try {
				Set services = (Set) cache.get(SERVICE_PREFIX + names[i],
						SERVICE_KEY);
				if (services == null)
					services = new HashSet();
				services.add(serviceUrl);
				cache.put(SERVICE_PREFIX + names[i], SERVICE_KEY, services);
				logger.info("added:" + serviceUrl);
			} catch (Exception ex) {
				logger.error("exception adding service:", ex);
			}
		}
	}

	private String makeUrl(String name) {
		return new StringBuffer().append("http://").append(localIpAddress)
				.append(":").append(localPort).append(urlQueryPrefix).append(
						name).toString();
	}

	public void setApplicationContext(ApplicationContext arg0)
			throws BeansException {
		context = arg0;
	}

	public ServicePublisher() throws Exception {
		super();
		localIpAddress = InetAddress.getLocalHost().getHostAddress().toString();
	}

	public void setLocalIpAddress(String localIpAddress) {
		this.localIpAddress = localIpAddress;
	}

	public void setLocalPort(String localPort) {
		this.localPort = localPort;
	}

	public void setUrlQueryPrefix(String urlQueryPrefix) {
		this.urlQueryPrefix = urlQueryPrefix;
	}

}
